"""
KOKAO Advanced Modules v31-v32

Stochastic Resonance, Rhythm Module, Energy Management.
Стохастический резонанс, Ритмический модуль, Энергоменеджмент.
"""

from kokao.modules.stochastic import StochasticResonance
from kokao.modules.rhythm import RhythmModule
from kokao.modules.energy import EnergyManager

__all__ = [
    "StochasticResonance",
    "RhythmModule", 
    "EnergyManager"
]
